@@ -534,6 +534,14 @@ def test_pairwise_distances_argmin_min(dtype):
 
     assert_array_equal(argmin_0, argmin_1)
 
+    # F-contiguous arrays must be supported and must return identical results.
+    argmin_C_contiguous = pairwise_distances_argmin(X, Y)
+    argmin_F_contiguous = pairwise_distances_argmin(
+        np.asfortranarray(X), np.asfortranarray(Y)
+    )
+
+    assert_array_equal(argmin_C_contiguous, argmin_F_contiguous)
+
 
 def _reduce_func(dist, start):
     return dist[:, :100]
